"""
show_position.py defines functions showing position device info.
"""

from Tkinter import *
import thread
import time


class show_position (Tk):
    """
    GUI class for position devices.
    """

    def __init__(self, player, index=0, speed=10, master=None):
        """
	create a separate window using Tkinter widgets.
	create a thread for periodic sensory data update.
	"""
	# save the configuration
	self.__player = player
	self.__device_index = index
	self.__update_speed = speed
	# create a gui
	Tk.__init__(self)
	self.title('Position : ' + str(self.__device_index))
	self.__position = Label(self, width=16, anchor=W)	# position
	self.__speed = Label(self, anchor=W)			# speed
	self.__stall = IntVar()					# stall
	# set layouts of widgets
	Label(self, text='Position : ').grid(row=0, column=0, sticky=E)
	self.__position.grid(row=0, column=1, sticky=W)
	Label(self, text='Speed : ').grid(row=1, column=0, sticky=E)
	self.__speed.grid(row=1, column=1, sticky=W)
	Checkbutton(self, variable=self.__stall, text='Stall').grid(
				row=2, column=0, columnspan=2, stick=W)
	self.grid()
	# start an update thread
	self.__update_flag = 1
	thread.start_new_thread(self.__update_thread, ())


    def configure(self, index=0, speed=10):
        """
	change the configuration on the fly.
	"""
	self.__device_index = index
	self.__update_speed = speed


    def close(self):
        """
	close the window, and kill it self.
	"""
	self.__update_flag = 0


    def __update_thread(self):
        """
	update sensory information.
	"""
	if self.__player.debug: print '<start a position update thread>'
	while self.__update_flag:
	    try:
	        # update odometry information
		x = self.__player.position[self.__device_index][0]
		y = self.__player.position[self.__device_index][1]
		self.__position.configure(text='(%d, %d, %d)' % \
		        self.__player.position[self.__device_index][0])
		# update speed info
		self.__speed.configure(text='(%d, %d, %d)' % \
		        self.__player.position[self.__device_index][1])
		# update stall info
		if self.__player.position[self.__device_index][2]: self.__stall.set(1)
		else: self.__stall.set(0)
		# slow down
		time.sleep(1.0 / self.__update_speed)
	    except KeyError:
	        print '<position:%d is not enabled yet>' % self.__device_index
		self.close()
	    except AttributeError:
		print '<invalid player object>'
	        self.close()
	    except TclError:
		self.close()
	if self.__player.debug: print '<the position update thread finishes>'

