"""
show_sonar.py defines functions showing sonar device info.
"""

from Tkinter import *
import thread
import time
import math


class show_sonar (Tk):
    """
    GUI class for sonar devices.
    """

    def __init__(self, player, index=0, speed=10, master=None):
        """
	create a separate window using Tkinter widgets.
	create a thread for periodic sensory data update.
	"""
	# save the configuration
	self.__player = player
	self.__device_index = index
	self.__update_speed = speed
	# create a gui
	Tk.__init__(self)
	self.title('Sonar : ' + str(self.__device_index))
	self.__win = Canvas(self, bg='black', width=400, height=400)
	# set layouts of widgets
	self.__win.grid(row=0, column=0, sticky=N+E+S+W)
	self.grid()
	# draw equidistance circles
	for idx in range(30,283,30):
	    self.__win.create_oval(200-idx, 200-idx, 200+idx, 200+idx, outline='gray40')
	# retrieve the sonar geometry
	geom = self.__player.get_sonar_geometry()
	# generate line items
	self.__lines = []
	for idx in range(len(geom)):
	    self.__lines.append(self.__win.create_line(200,200, 200,200, fill='yellow', width=2))
	# pre-calculation for later use
	self.__sin = []
	self.__cos = []
	self.__pos = []
	for i in range(len(geom)):
	    self.__sin.append(math.sin(math.pi/180 * (geom[i][2]+90)) / 1000 * 30)
	    self.__cos.append(math.cos(math.pi/180 * (geom[i][2]+90)) / 1000 * 30)
	    self.__pos.append((200 - geom[i][1] * 30/1000.0,
	    		       200 - geom[i][0] * 30/1000.0))
	# start an update thread
	self.__update_flag = 1
	thread.start_new_thread(self.__update_thread, ())


    def configure(self, index=0, speed=10):
        """
	change the configuration on the fly.
	"""
	self.__device_index = index
	self.__update_speed = speed


    def close(self):
        """
	close the window, and kill it self.
	"""
	self.__update_flag = 0


    def __update_thread(self):
        """
	update sensory information.
	"""
	if self.__player.debug: print '<start a sonar update thread>'
	while self.__update_flag:
	    try:
		# update canvas window
		for idx in range(len(self.__player.sonar[self.__device_index])):
		    self.__win.coords(self.__lines[idx],
			self.__pos[idx][0], self.__pos[idx][1],
			self.__pos[idx][0] +
			    self.__player.sonar[self.__device_index][idx]*self.__cos[idx],
			self.__pos[idx][1] -
			    self.__player.sonar[self.__device_index][idx]*self.__sin[idx])
		# slow down
		time.sleep(1.0 / self.__update_speed)
	    except KeyError:
	        print '<sonar:%d is not enabled yet>' % self.__device_index
		self.close()
	    except AttributeError:
		print '<invalid player object>'
	        self.close()
	    except TclError:
		self.close()
	if self.__player.debug: print '<the sonar update thread finishes>'

