"""
show_wifi.py defines functions showing wifi device info.
"""

from Tkinter import *
import thread
import time


class show_wifi (Tk):
    """
    GUI class for wifi devices.
    """

    def __init__(self, player, index=0, speed=10, master=None):
        """
	create a separate window using Tkinter widgets.
	create a thread for periodic sensory data update.
	"""
	# save the configuration
	self.__player = player
	self.__device_index = index
	self.__update_speed = speed
	# create a gui
	Tk.__init__(self)
	self.title('Wifi : ' + str(self.__device_index))
	self.__ip = []
	self.__link = []
	self.__level = []
	self.__noise = []
	for idx in range(len(self.__player.wifi[self.__device_index])):
	    Label(self, text='IP : ').grid(row=idx*4, column=0, sticky=W)
	    self.__ip.append(Label(self))
	    self.__ip[idx].grid(row=idx*4, column=1, columnspan=2, sticky=W)
	    Label(self, text='- Link Quality : ').grid(row=idx*4+1, column=1, sticky=W)
	    self.__link.append(Label(self))
	    self.__link[idx].grid(row=idx*4+1, column=2, sticky=W)
	    Label(self, text='- Level : ').grid(row=idx*4+2, column=1, sticky=W)
	    self.__level.append(Label(self))
	    self.__level[idx].grid(row=idx*4+2, column=2, sticky=W)
	    Label(self, text='- Noise : ').grid(row=idx*4+3, column=1, sticky=W)
	    self.__noise.append(Label(self))
	    self.__noise[idx].grid(row=idx*4+3, column=2, sticky=W)
	self.grid()
	# start an update thread
	self.__update_flag = 1
	thread.start_new_thread(self.__update_thread, ())


    def configure(self, index=0, speed=10):
        """
	change the configuration on the fly.
	"""
	self.__device_index = index
	self.__update_speed = speed


    def close(self):
        """
	close the window, and kill it self.
	"""
	self.__update_flag = 0


    def __binary_string(self, value):
        """
	convert an integer value to a binary string.
	"""
	binstr = ''
	for idx in range(8):
	    binstr = str(value & 0x01) + binstr
	    value >>= 1
	return binstr


    def __update_thread(self):
        """
	update sensory information.
	"""
	if self.__player.debug: print '<start a wifi update thread>'
	while self.__update_flag:
	    try:
	        # update info
		for idx in range(len(self.__player.wifi[self.__device_index])):
		    self.__ip[idx].configure(text=
		        self.__player.wifi[self.__device_index][idx][0])
		    self.__link[idx].configure(text=
		        str(self.__player.wifi[self.__device_index][idx][1]))
		    self.__level[idx].configure(text=
		        str(self.__player.wifi[self.__device_index][idx][2]))
		    self.__noise[idx].configure(text=
		        str(self.__player.wifi[self.__device_index][idx][3]))
		# slow down
		time.sleep(1.0 / self.__update_speed)
	    except KeyError:
	        print '<wifi:%d is not enabled yet>' % self.__device_index
		self.close()
	    except AttributeError:
		print '<invalid player object>'
	        self.close()
	    except TclError:
		self.close()
	if self.__player.debug: print '<the wifi update thread finishes>'

